--------------------------------------------------------------------------------
-- GLOBALS
--------------------------------------------------------------------------------

-- NOTE: IF THE MAXIMUM LEVEL IN ALLODS CHANGES, PLEASE UPDATE THIS VALUE
-- TO REFLECT THE NEW HIGHEST POSSIBLE LEVEL.
-- Previous attempts where made to compute this automatically but changes made
-- to the API during 3.0.2 have rendered the ability to workout (reliablity) what
-- the maximum possible level is. Therefore I have had to set this as a static value.

Global( "AO_MAX_LEVEL", 51 )
Global( "wtStatPanel", nil )
Global( "labStatNames", nil)
Global( "labStatValues", nil)

Global("onEvents", {} )
Global("onReactions", {} )

Global( "localization", "eng" )

--------------------------------------------------------------------------------
-- EVENT HANDLERS
--------------------------------------------------------------------------------


onEvents[ "EVENT_AVATAR_CREATED" ] = function( params )
	-- Localization detection:
	localization = GetGameLocalization()
	if not Locales[ localization ] or not StatNames[ localization ] then
		localization = "eng"
	end
	
	-- Applying localization:
	labStatNames:SetVal( "Score", GetTextLocalized( "Score" ) )
	labStatNames:SetVal( "PlayerScore", GetTextLocalized( "PlayerScore" ) )
	for j, Stat in StatNames[localization] do
		labStatNames:SetVal( "Stat"..j, userMods.ToWString( StatNames[ localization ][ j ] ) )
	end
	
	UpdateAddon()
	
	onEvents[ "SCRIPT_ADDON_INFO_REQUEST" ]( { target = common.GetAddonName() } )
end


onEvents[ "EVENT_INSPECT_FINISHED" ] = function( params )
	wtStatPanel:Show( false )
end


onEvents[ "EVENT_INSPECT_STARTED" ] = function( params )
	local PlayerID = avatar.GetId()
	local PlayerLevel = unit.GetLevel( PlayerID )
	local PlayerClassInfo = unit.GetClass( PlayerID )
	local PlayerClass = PlayerClassInfo.className
	local PlayerBonusStat 
	local PlayerBonusStatValue
	
	local TargetID = unit.GetPrimaryTarget( PlayerID )
	local TargetBonusStat
	local TargetBonusStatValue
	
	local TargetStats = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 }
	local PlayerStats = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 }
	TargetStats[0] = 0
	PlayerStats[0] = 0
	
	local TargetResult = 0 
	local PlayerResult = 0
	
	-- Used to determine the color of Velhuru
	local TargetGearScore = 0
	local PlayerGearScore = 0
	
	if TargetID and unit.IsPlayer( TargetID ) then
		local TargetLevel = unit.GetLevel( TargetID )
		local TargetClassInfo = unit.GetClass( TargetID )
		local TargetClass = TargetClassInfo.className
		
		local targetItems, playerItems
		if social.GetFriendInfo then -- AO 1.1.04+
			playerItems = unit.GetEquipmentItemIds( PlayerID, ITEM_CONT_EQUIPMENT )
			targetItems = unit.GetEquipmentItemIds( TargetID, ITEM_CONT_EQUIPMENT )
		else -- AO 1.1.02/03
			playerItems = unit.GetEquipmentItemIds( PlayerID )
			targetItems = unit.GetEquipmentItemIds( TargetID )
		end
		
		-- Calculate a Level Base - Target level to the power of two.
		TargetResult = (math.pow(TargetLevel, 2))
		PlayerResult = (math.pow(PlayerLevel, 2))
		
		-- Calculate Target Patron Bonus
		local patronInfo = unit.GetFairyInfo ( TargetID )
		if patronInfo then
			local patronRank = 		patronInfo.rank
			local bonusStat = 		patronInfo.bonusStat
			local bonusStatValue = 	patronInfo.bonusStatValue
			
			-- Reset all Stat Colors to Default
			for j, Stat in StatNames[localization] do
				if ClassStatType[TargetClass][j] == 2 then
					labStatNames:SetClassVal("Class"..j, "StatBuffed")
				else
					labStatNames:SetClassVal("Class"..j, "StatNormal")
				end
				labStatValues:SetClassVal("Class"..j, "StatBuffed")
			end
			
			-- Set Patron Stat color on the name and stat
			labStatNames:SetClassVal("Class"..bonusStat, "LogColorYellow")
			labStatValues:SetClassVal("Class"..bonusStat, "LogColorYellow")
		end
		
		TargetResult = TargetResult + CalculateVelhuruRating( TargetID, TargetLevel, targetItems, TargetStats )
		PlayerResult = PlayerResult + CalculateVelhuruRating( PlayerID, PlayerLevel, playerItems, PlayerStats )
		
		TargetGearScore = CalculateGearScore( targetItems )
		PlayerGearScore = CalculateGearScore( playerItems )
		
		-- Output the Final Stat value to the GUI
		for j, Stat in StatNames[localization] do
			local outputString = "";
			if PlayerStats[j] ~= 0 and TargetStats[j] ~= 0 and WithinPlayerLevel( PlayerLevel, TargetLevel ) and isSameClass( PlayerID, TargetID ) then
				local delta = math.floor(((TargetStats[j] / PlayerStats[j]) * 100) - 100)
				if delta and delta > 0 then
					labStatValues:SetClassVal("Class"..j, "tip_green")
				elseif delta and delta < 0 then
					labStatValues:SetClassVal("Class"..j, "tip_red")
				else
					labStatValues:SetClassVal("Class"..j, "tip_yellow")
				end
				outputString = string.format("%d", delta).."%"
				
				-- Set the Delta values
				labStatValues:SetVal( "Stat"..j, common.FormatInt( TargetStats[j], "%d" ) )
				labStatValues:SetVal( "Delta"..j, userMods.ToWString( outputString ) )
			else
				labStatValues:SetVal( "Stat"..j,  userMods.ToWString(" ") )
				labStatValues:SetVal( "Delta"..j, common.FormatInt( TargetStats[j], "%d" )  )
			end
			
			
		end
		
		-- Set the Score Values for both Target and Player
		labStatValues:SetVal( "Score", common.FormatInt( math.floor( TargetResult ), "%d" ) )
		labStatValues:SetVal( "PlayerScore", common.FormatInt( math.floor( PlayerResult ), "%d" ) )
		
		-- Set the Color Quality of the
		SetVelhuruColor( TargetGearScore, true )
		SetVelhuruColor( PlayerGearScore, false )
		
		wtStatPanel:Show( true )
	else
		wtStatPanel:Show( false )
	end
end

function CalculateVelhuruRating( PlayerID, PlayerLevel, PlayerItems, PlayerStats)
	local VelhuruRating = 0
	
	-- Calculate Player Patron Bonus
	local patronInfo = unit.GetFairyInfo ( PlayerID )
	if patronInfo then
		local bonusStat = 		patronInfo.bonusStat
		local bonusStatValue =	patronInfo.bonusStatValue
		-- Add the Patron Stat Bonus
		PlayerStats[bonusStat] = PlayerStats[bonusStat] + bonusStatValue
	end
	
	-- Calculate the Equipment Stats
	for i, ItemId in PlayerItems do
		local Item = avatar.GetItemInfo(ItemId)
		
		-- Calculate Targets Stats
		local ItemStat = avatar.GetItemBonus(ItemId)
		for j, Stat in ItemStat.innateStats do
			PlayerStats[j] = PlayerStats[j] + Stat.base
		end
				
		if IsValidItem ( Item ) then
			local ItemValue			
			if Item.level > 45 then
				ItemValue = (math.pow(Item.level, 2) * (Item.quality + (Item.level - 45))) / 100
			else
				ItemValue = (math.pow(Item.level, 2) * (Item.quality)) / 100
			end
			
			-- Balance 2h vs 1h + Off
			if Item.dressSlot == DRESS_SLOT_TWOHANDED or Item.dressSlot == DRESS_SLOT_DUALWIELD then
				ItemValue = ItemValue * 2
			end
			
			VelhuruRating = VelhuruRating + ItemValue
		end
	end

	-- Multiply just to get a nice high number.
	VelhuruRating = VelhuruRating * 2.1
	
	return VelhuruRating
end

function CalculateGearScore( targetItems )
	local GearScore = 0
	local ItemValue = 0
	local ItemCount = 17;	
		
	-- Calculate the Target Equipment Stats
	for i, ItemId in targetItems do
		local Item = avatar.GetItemInfo(ItemId)
		if IsValidItem ( Item ) then
			ItemValue = Item.quality + (Item.level - 45);
			if Item.dressSlot == DRESS_SLOT_TWOHANDED or Item.dressSlot == DRESS_SLOT_DUALWIELD then
				ItemValue = ItemValue * 2
			end
			GearScore = GearScore + ItemValue
		end
	end
	
	if ItemCount == 0 then
		ItemCount = 1
	end
	
	if GearScore == 0 then
		GearScore = 1
	end
	
	-- Their is a total of 18 items calculated in quality, but because the 2nd earring
	-- slot does not have a mythical quality or that a lot of players might not have a mythic quality 2nd earring,
	-- therefore I reduced the item count by 1 to allow people with full mythic max level items (besides the 2nd ear)
	-- to still acheive a mythic quality gear score. At least that is the intention
	return GearScore / ItemCount
end

function IsValidItem( Item ) 
	if Item.dressSlot == DRESS_SLOT_HELM or
	   Item.dressSlot == DRESS_SLOT_ARMOR or
	   Item.dressSlot == DRESS_SLOT_PANTS or
	   Item.dressSlot == DRESS_SLOT_BOOTS or
	   Item.dressSlot == DRESS_SLOT_MANTLE or
	   Item.dressSlot == DRESS_SLOT_GLOVES or
	   Item.dressSlot == DRESS_SLOT_BRACERS or
	   Item.dressSlot == DRESS_SLOT_BELT or
	   Item.dressSlot == DRESS_SLOT_RING1 or
	   Item.dressSlot == DRESS_SLOT_RING2 or
	   Item.dressSlot == DRESS_SLOT_RING or
	   Item.dressSlot == DRESS_SLOT_EARRING1 or
	   Item.dressSlot == DRESS_SLOT_EARRINGS or
	   Item.dressSlot == DRESS_SLOT_NECKLACE or
	   Item.dressSlot == DRESS_SLOT_CLOAK or
	   Item.dressSlot == DRESS_SLOT_SHIRT or
	   Item.dressSlot == DRESS_SLOT_MAINHAND or
	   Item.dressSlot == DRESS_SLOT_OFFHAND or
	   Item.dressSlot == DRESS_SLOT_RANGED or
	   Item.dressSlot == DRESS_SLOT_EARRING2 or
	   Item.dressSlot == DRESS_SLOT_TWOHANDED or
	   Item.dressSlot == DRESS_SLOT_DUALWIELD then
	   
		return true
	else
		return false;
	end
end

----------------------------------------------------------
-- Sets the designated Stat Title and Number to a 
-- Quality Color. The Stat ranges are setup based on
-- mathimatical formulas to determine maximum equipment
-- points an individual can achieve.
----------------------------------------------------------
function SetVelhuruColor( Value , isTarget ) 
	local Class
	if isTarget then
		Class = "Score"
	else
		Class = "PlayerScore"
	end

	labStatValues:SetClassVal(Class, getVelhuruColor(Value))
	labStatNames:SetClassVal(Class, getVelhuruColor(Value))	
end

function getVelhuruColor( Value )
	if avatar.IsExist() then
		-- Basically whats going on here, its taking away the lowest possible level to begin
		-- gearing (45) then turning this into a modifier on the existing gear quality range.
		
		local qualityModifier = (((AO_MAX_LEVEL) - 45))
				
		if Value >= ( ITEM_QUALITY_RELIC + qualityModifier ) then
			return "Relic"
		elseif Value >= ( ITEM_QUALITY_LEGENDARY + qualityModifier ) and Value < ( ITEM_QUALITY_RELIC + qualityModifier ) then
			return "Legendary"
		elseif Value >= ( ITEM_QUALITY_EPIC + qualityModifier ) and Value < ( ITEM_QUALITY_LEGENDARY + qualityModifier ) then 
			return "Epic"
		elseif Value >= ( ITEM_QUALITY_RARE + qualityModifier ) and Value < ( ITEM_QUALITY_EPIC + qualityModifier ) then 
			return "Rare"
		elseif Value >= ( ITEM_QUALITY_UNCOMMON + qualityModifier ) and Value < ( ITEM_QUALITY_RARE + qualityModifier ) then 
			return "Uncommon"
		elseif Value >= ( ITEM_QUALITY_COMMON + qualityModifier ) and Value < ( ITEM_QUALITY_UNCOMMON + qualityModifier ) then 
			return "Common"
		else
			return "Goods"
		end
	else
		return "Goods"
	end

end

-----------------------------------------------------------------
-- Checks to see if the player being inspected is within a 6 
-- level difference. If not then the Delta's will not be displayed.
-- This is help stop large numbers showing in gear difference.
-----------------------------------------------------------------
function WithinPlayerLevel( PlayerLevel, TargetLevel )
	local min = PlayerLevel - 3
	local max = PlayerLevel + 3
	
	if TargetLevel <= max and TargetLevel >= min then
		return true;
	else
		return false;
	end
end

function isSameClass( PlayerID, TargetID )
	local PlayerClass = unit.GetClass(PlayerID)
	local TargetClass = unit.GetClass(TargetID)
	
	return PlayerClass.className == TargetClass.className
end

------------------------------------------------------------
-- This function is used for AO Panel to display a players
-- Velhuru in the Panel.
------------------------------------------------------------
function getPlayerVelhuru ( PlayerID, isAvatar ) 
	if PlayerID and unit.IsPlayer( PlayerID ) then
		local PlayerLevel = unit.GetLevel( PlayerID )
		local PlayerClassInfo = unit.GetClass( PlayerID )
		local PlayerClass = PlayerClassInfo.className
	
		local playerItems
		if social.GetFriendInfo then -- AO 1.1.04+
			playerItems = unit.GetEquipmentItemIds( PlayerID, ITEM_CONT_EQUIPMENT )
		else -- AO 1.1.02/03
			playerItems = unit.GetEquipmentItemIds( PlayerID )
		end
		
		local PlayerStats = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 }
		PlayerStats[0] = 0
		local PlayerResult = (math.pow(PlayerLevel, 2))
		
		PlayerResult = PlayerResult + CalculateVelhuruRating( PlayerID, PlayerLevel, playerItems, PlayerStats )
		
		return PlayerResult
	end
	return 0
end


onEvents[ "SCRIPT_ADDON_INFO_REQUEST" ] = function( params )
	if params.target == common.GetAddonName() then
		userMods.SendEvent( "SCRIPT_ADDON_INFO_RESPONSE", {
			sender = params.target,
			desc = userMods.FromWString( GetTextLocalized( "Description" ) ),
		} )
	end
end

onEvents[ "SCRIPT_ADDON_MEM_REQUEST" ] = function( params )
	if params.target == common.GetAddonName() then
		userMods.SendEvent( "SCRIPT_ADDON_MEM_RESPONSE", { sender = params.target, memUsage = gcinfo() } )
	end
end

onEvents[ "SCRIPT_TOGGLE_UI" ] = function( params )
	if not params.visible then
		wtStatPanel:Show( false )
	end
end


--------------------------------------------------------------------------------
-- REACTION HANDLERS
--------------------------------------------------------------------------------

onEvents [ "AOPANEL_START" ] = function( params )
	SendAddon()
end

onEvents [ "EVENT_UNIT_EQUIPMENT_CHANGED" ] = function( params )
	UpdateAddon()
end

function SendAddon()
	if avatar.IsExist() then
		local velhuru = math.floor( getPlayerVelhuru( avatar.GetId(), true) )
		local playerItems
		if social.GetFriendInfo then -- AO 1.1.04+
			playerItems = unit.GetEquipmentItemIds( avatar.GetId(), ITEM_CONT_EQUIPMENT )
		else -- AO 1.1.02/03
			playerItems = unit.GetEquipmentItemIds( avatar.GetId() )
		end
		local gearScore = CalculateGearScore( playerItems )
		
		local SetVal = { val1 = userMods.ToWString( "Velhuru: "..string.format("%d", velhuru)), class1 = getVelhuruColor( gearScore ) }
		local params = { header = SetVal, ptype = "text", size = 130 } 
		userMods.SendEvent( "AOPANEL_SEND_ADDON", { name = "Inspect Stats Velhuru", sysName = "InspectStatsVelhuru", param = params } )
	end
end

function UpdateAddon(params)
	if avatar.IsExist() then
		local velhuru = math.floor( getPlayerVelhuru( avatar.GetId(), true) )
		local playerItems
		if social.GetFriendInfo then -- AO 1.1.04+
			playerItems = unit.GetEquipmentItemIds( avatar.GetId(), ITEM_CONT_EQUIPMENT )
		else -- AO 1.1.02/03
			playerItems = unit.GetEquipmentItemIds( avatar.GetId() )
		end
		local gearScore = CalculateGearScore( playerItems )
		
		local SetVal = { val1 = userMods.ToWString("Velhuru: "..string.format("%d", velhuru)), class1 = getVelhuruColor( gearScore ) }
		userMods.SendEvent( "AOPANEL_UPDATE_ADDON", { sysName = "InspectStatsVelhuru", header = SetVal } )
	end
end

function GetTableSize( t )
	if not t then
		return 0
	end
	local count = 0
	for k, v in t do
		count = count + 1
	end
	return count
end

--------------------------------------------------------------------------------
-- INITIALIZATION
--------------------------------------------------------------------------------



function Init()
	-- Panels
	wtStatPanel = mainForm:GetChildChecked( "StatPanel", false )
	
	labStatNames = wtStatPanel:GetChildChecked( "labStatNames", false )
	labStatValues = wtStatPanel:GetChildChecked( "labStatValues", false )
	
	-- Events
	RegisterEventHandlers( onEvents )
	
	-- Reactions
	RegisterReactionHandlers( onReactions )
	
	-- Backward compatibility with pre-1.1.04 versions:
	if not social.GetFriendInfo then
		mainForm:SetPriority( 900 )
	end
	
	-- For the case addon was reloaded by AddonManager:
	if avatar.IsExist() then
		onEvents[ "EVENT_AVATAR_CREATED" ]()
	end
end

--------------------------------------------------------------------------------
Init()
--------------------------------------------------------------------------------
